-- This file contains the behavioral model of the LMS update block 

--Library declaraions
library IEEE;
use IEEE.Std_logic_1164.ALL;

library COMPASS_LIB;
use COMPASS_LIB.COMPASS.ALL;
--compass compile_off
--use COMPASS_LIB.COMPASS_ETC.ALL;
--compass compile_on

entity LMS is 
 port ( Clk    : in std_logic;
        Reset  : in std_logic;
        sign_i : in std_logic;
        sign_q : in std_logic;
        Ei     : in std_logic_vector(15 downto 0);      -- real error 
        Eq     : in std_logic_vector(15 downto 0);     -- imaginary error
        Ci     : out std_logic_vector(7 downto 0);  -- real coefficient 
        Cq     : out std_logic_vector(7 downto 0) );  -- imaginary coefficient 
end LMS;        

architecture behav of LMS is

signal offset_ii: std_logic_vector(15 downto 0);
signal offset_iq: std_logic_vector(15 downto 0);
signal offset_qq: std_logic_vector(15 downto 0);
signal offset_qi: std_logic_vector(15 downto 0);

signal offset_i : std_logic_vector(15 downto 0);
signal offset_q : std_logic_vector(15 downto 0);

signal ci_i     : std_logic_vector(23 downto 0);
signal cq_i     : std_logic_vector(23 downto 0);

signal sum_i    : std_logic_vector(24 downto 0);
signal sum_q    : std_logic_vector(24 downto 0);

signal dummy_i  : std_logic_vector(24 downto 0);
signal dummy_q  : std_logic_vector(24 downto 0);

begin

  offset_ii <= Ei when  sign_i = '1' else not(Ei);
  offset_iq <= Ei when  sign_q = '0' else not(Ei);
  offset_qq <= Eq when  sign_q = '1' else not(Eq);
  offset_qi <= Eq when  sign_i = '1' else not(Eq);

offset_i <= offset_ii + offset_qq; 
offset_q <= offset_iq + offset_qi; 

dummy_i <= "000000000" & offset_i;
dummy_q <= "000000000" & offset_q;

sum_i <= dummy_i + ci_i;
sum_q <= dummy_q + cq_i;

ci <= ci_i(23 downto 16);
cq <= cq_i(23 downto 16);

process(reset,clk)
  
   begin
    if (reset = '0') then 
       ci_i <= "000101010000000000000000";
       cq_i <= "000101110000000000000000";

    elsif (clk'event and clk = '1') then
       
       ci_i <= sum_i(23 downto 0);
       cq_i <= sum_q(23 downto 0);

    end if;
  end process;     
       
end behav;
      
